/*****************************************************************************************
* Copyright (c) 2006 Hewlett-Packard Development Company, L.P.
* Permission is hereby granted, free of charge, to any person obtaining a copy of 
* this software and associated documentation files (the "Software"), to deal in 
* the Software without restriction, including without limitation the rights to use, 
* copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the 
* Software, and to permit persons to whom the Software is furnished to do so, 
* subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
* INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
* CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
* OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
*****************************************************************************************/

/************************************************************************
 * SVN MACROS
 *
 * $LastChangedDate: 2007-05-14 13:35:33 +0530 (Mon, 14 May 2007) $
 * $Revision: 92 $
 * $Author: sharmnid $
 *
 ************************************************************************/
/************************************************************************
 * FILE DESCR: Implementation of LTKShapeFeatureExtractor which would be used as
 *			   a place holder in LTKShapeFeatureExtractor 
 *			   for anyone of the implemented feature extractor 
 *			   which is derived from this class
 *
 * CONTENTS: 
 *
 * AUTHOR:     
 *
 * DATE:       
 * CHANGE HISTORY:
 * Author		Date			Description of change
 ************************************************************************/

#include "LTKShapeFeatureExtractor.h"
#include "LTKErrorsList.h"

/**********************************************************************************
* AUTHOR		: Tarun Madan
* DATE			: 07-Aug-2007
* NAME			: convertToTraceGroup
* DESCRIPTION	:
* ARGUMENTS		: 
* RETURNS		: 
* NOTES			:
* CHANGE HISTROY
* Author			Date				Description of change
*************************************************************************************/
int LTKShapeFeatureExtractor::convertFeatVecToTraceGroup(
                                 const vector<LTKShapeFeaturePtr>& shapeFeature,
                                       LTKTraceGroup& outTraceGroup)
{
	
	return(EFTR_RPRCLASS_NOIMPLEMENTATION);
	
}