/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardselectionlistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardselectionlistmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardselectionlistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QVirtualKeyboardSelectionListModel_t {
    QByteArrayData data[26];
    char stringdata0[336];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QVirtualKeyboardSelectionListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QVirtualKeyboardSelectionListModel_t qt_meta_stringdata_QVirtualKeyboardSelectionListModel = {
    {
QT_MOC_LITERAL(0, 0, 34), // "QVirtualKeyboardSelectionList..."
QT_MOC_LITERAL(1, 35, 12), // "countChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 17), // "activeItemChanged"
QT_MOC_LITERAL(4, 67, 5), // "index"
QT_MOC_LITERAL(5, 73, 12), // "itemSelected"
QT_MOC_LITERAL(6, 86, 20), // "selectionListChanged"
QT_MOC_LITERAL(7, 107, 4), // "Type"
QT_MOC_LITERAL(8, 112, 4), // "type"
QT_MOC_LITERAL(9, 117, 30), // "selectionListActiveItemChanged"
QT_MOC_LITERAL(10, 148, 10), // "selectItem"
QT_MOC_LITERAL(11, 159, 10), // "removeItem"
QT_MOC_LITERAL(12, 170, 6), // "dataAt"
QT_MOC_LITERAL(13, 177, 4), // "Role"
QT_MOC_LITERAL(14, 182, 4), // "role"
QT_MOC_LITERAL(15, 187, 5), // "count"
QT_MOC_LITERAL(16, 193, 17), // "WordCandidateList"
QT_MOC_LITERAL(17, 211, 7), // "Display"
QT_MOC_LITERAL(18, 219, 11), // "DisplayRole"
QT_MOC_LITERAL(19, 231, 20), // "WordCompletionLength"
QT_MOC_LITERAL(20, 252, 24), // "WordCompletionLengthRole"
QT_MOC_LITERAL(21, 277, 10), // "Dictionary"
QT_MOC_LITERAL(22, 288, 19), // "CanRemoveSuggestion"
QT_MOC_LITERAL(23, 308, 14), // "DictionaryType"
QT_MOC_LITERAL(24, 323, 7), // "Default"
QT_MOC_LITERAL(25, 331, 4) // "User"

    },
    "QVirtualKeyboardSelectionListModel\0"
    "countChanged\0\0activeItemChanged\0index\0"
    "itemSelected\0selectionListChanged\0"
    "Type\0type\0selectionListActiveItemChanged\0"
    "selectItem\0removeItem\0dataAt\0Role\0"
    "role\0count\0WordCandidateList\0Display\0"
    "DisplayRole\0WordCompletionLength\0"
    "WordCompletionLengthRole\0Dictionary\0"
    "CanRemoveSuggestion\0DictionaryType\0"
    "Default\0User"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QVirtualKeyboardSelectionListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       1,   88, // properties
       3,   92, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    1,   60,    2, 0x06 /* Public */,
       5,    1,   63,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   66,    2, 0x09 /* Protected */,
       9,    2,   69,    2, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
      10,    1,   74,    2, 0x02 /* Public */,
      11,    1,   77,    2, 0x02 /* Public */,
      12,    2,   80,    2, 0x02 /* Public */,
      12,    1,   85,    2, 0x22 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Int,    8,    4,

 // methods: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::QVariant, QMetaType::Int, 0x80000000 | 13,    4,   14,
    QMetaType::QVariant, QMetaType::Int,    4,

 // properties: name, type, flags
      15, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,

 // enums: name, alias, flags, count, data
       7,    7, 0x2,    1,  107,
      13,   13, 0x2,    6,  109,
      23,   23, 0x2,    2,  121,

 // enum data: key, value
      16, uint(QVirtualKeyboardSelectionListModel::Type::WordCandidateList),
      17, uint(QVirtualKeyboardSelectionListModel::Role::Display),
      18, uint(QVirtualKeyboardSelectionListModel::Role::DisplayRole),
      19, uint(QVirtualKeyboardSelectionListModel::Role::WordCompletionLength),
      20, uint(QVirtualKeyboardSelectionListModel::Role::WordCompletionLengthRole),
      21, uint(QVirtualKeyboardSelectionListModel::Role::Dictionary),
      22, uint(QVirtualKeyboardSelectionListModel::Role::CanRemoveSuggestion),
      24, uint(QVirtualKeyboardSelectionListModel::DictionaryType::Default),
      25, uint(QVirtualKeyboardSelectionListModel::DictionaryType::User),

       0        // eod
};

void QVirtualKeyboardSelectionListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QVirtualKeyboardSelectionListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->activeItemChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->itemSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->selectionListChanged((*reinterpret_cast< Type(*)>(_a[1]))); break;
        case 4: _t->selectionListActiveItemChanged((*reinterpret_cast< Type(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->selectItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->removeItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: { QVariant _r = _t->dataAt((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Role(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 8: { QVariant _r = _t->dataAt((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QVirtualKeyboardSelectionListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardSelectionListModel::countChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardSelectionListModel::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardSelectionListModel::activeItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardSelectionListModel::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardSelectionListModel::itemSelected)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QVirtualKeyboardSelectionListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QVirtualKeyboardSelectionListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QVirtualKeyboardSelectionListModel.data,
    qt_meta_data_QVirtualKeyboardSelectionListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QVirtualKeyboardSelectionListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardSelectionListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QVirtualKeyboardSelectionListModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QVirtualKeyboardSelectionListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardSelectionListModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QVirtualKeyboardSelectionListModel::activeItemChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QVirtualKeyboardSelectionListModel::itemSelected(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
