/****************************************************************************
** Meta object code from reading C++ file 'hunspellworker_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../kde-qtvirtualkeyboard/src/plugins/hunspell/hunspellinputmethod/hunspellworker_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hunspellworker_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellTask_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellTask = {
    {
QT_MOC_LITERAL(0, 0, 31) // "QtVirtualKeyboard::HunspellTask"

    },
    "QtVirtualKeyboard::HunspellTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellTask::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellTask.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask_t {
    QByteArrayData data[4];
    char stringdata0[65];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask = {
    {
QT_MOC_LITERAL(0, 0, 45), // "QtVirtualKeyboard::HunspellLo..."
QT_MOC_LITERAL(1, 46, 9), // "completed"
QT_MOC_LITERAL(2, 56, 0), // ""
QT_MOC_LITERAL(3, 57, 7) // "success"

    },
    "QtVirtualKeyboard::HunspellLoadDictionaryTask\0"
    "completed\0\0success"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellLoadDictionaryTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HunspellLoadDictionaryTask *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->completed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HunspellLoadDictionaryTask::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HunspellLoadDictionaryTask::completed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellLoadDictionaryTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellLoadDictionaryTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellLoadDictionaryTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadDictionaryTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellLoadDictionaryTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::HunspellLoadDictionaryTask::completed(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask_t {
    QByteArrayData data[1];
    char stringdata0[48];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask = {
    {
QT_MOC_LITERAL(0, 0, 47) // "QtVirtualKeyboard::HunspellBu..."

    },
    "QtVirtualKeyboard::HunspellBuildSuggestionsTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellBuildSuggestionsTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellBuildSuggestionsTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellBuildSuggestionsTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellBuildSuggestionsTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellBuildSuggestionsTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellBuildSuggestionsTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask_t {
    QByteArrayData data[6];
    char stringdata0[114];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask = {
    {
QT_MOC_LITERAL(0, 0, 48), // "QtVirtualKeyboard::HunspellUp..."
QT_MOC_LITERAL(1, 49, 17), // "updateSuggestions"
QT_MOC_LITERAL(2, 67, 0), // ""
QT_MOC_LITERAL(3, 68, 32), // "QSharedPointer<HunspellWordList>"
QT_MOC_LITERAL(4, 101, 8), // "wordList"
QT_MOC_LITERAL(5, 110, 3) // "tag"

    },
    "QtVirtualKeyboard::HunspellUpdateSuggestionsTask\0"
    "updateSuggestions\0\0QSharedPointer<HunspellWordList>\0"
    "wordList\0tag"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellUpdateSuggestionsTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    4,    5,

       0        // eod
};

void QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HunspellUpdateSuggestionsTask *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateSuggestions((*reinterpret_cast< const QSharedPointer<HunspellWordList>(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HunspellUpdateSuggestionsTask::*)(const QSharedPointer<HunspellWordList> & , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HunspellUpdateSuggestionsTask::updateSuggestions)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellUpdateSuggestionsTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellUpdateSuggestionsTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellUpdateSuggestionsTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellUpdateSuggestionsTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellUpdateSuggestionsTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::HunspellUpdateSuggestionsTask::updateSuggestions(const QSharedPointer<HunspellWordList> & _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask_t {
    QByteArrayData data[1];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask = {
    {
QT_MOC_LITERAL(0, 0, 38) // "QtVirtualKeyboard::HunspellAd..."

    },
    "QtVirtualKeyboard::HunspellAddWordTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellAddWordTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellAddWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellAddWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellAddWordTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellAddWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellAddWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellAddWordTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellAddWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask_t {
    QByteArrayData data[1];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask = {
    {
QT_MOC_LITERAL(0, 0, 41) // "QtVirtualKeyboard::HunspellRe..."

    },
    "QtVirtualKeyboard::HunspellRemoveWordTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellRemoveWordTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellRemoveWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellRemoveWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellRemoveWordTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellRemoveWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellRemoveWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellRemoveWordTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellRemoveWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask_t {
    QByteArrayData data[1];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask = {
    {
QT_MOC_LITERAL(0, 0, 43) // "QtVirtualKeyboard::HunspellLo..."

    },
    "QtVirtualKeyboard::HunspellLoadWordListTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellLoadWordListTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellLoadWordListTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellLoadWordListTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellLoadWordListTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellLoadWordListTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellLoadWordListTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellLoadWordListTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellLoadWordListTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask_t {
    QByteArrayData data[1];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask = {
    {
QT_MOC_LITERAL(0, 0, 43) // "QtVirtualKeyboard::HunspellSa..."

    },
    "QtVirtualKeyboard::HunspellSaveWordListTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellSaveWordListTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellSaveWordListTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellSaveWordListTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellSaveWordListTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellSaveWordListTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellSaveWordListTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellSaveWordListTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellSaveWordListTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask_t {
    QByteArrayData data[1];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask = {
    {
QT_MOC_LITERAL(0, 0, 41) // "QtVirtualKeyboard::HunspellFi..."

    },
    "QtVirtualKeyboard::HunspellFilterWordTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellFilterWordTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellFilterWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellFilterWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellFilterWordTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellFilterWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellFilterWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellFilterWordTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellFilterWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask_t {
    QByteArrayData data[1];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask_t qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask = {
    {
QT_MOC_LITERAL(0, 0, 40) // "QtVirtualKeyboard::HunspellBo..."

    },
    "QtVirtualKeyboard::HunspellBoostWordTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellBoostWordTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellBoostWordTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellBoostWordTask::staticMetaObject = { {
    QMetaObject::SuperData::link<HunspellTask::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask.data,
    qt_meta_data_QtVirtualKeyboard__HunspellBoostWordTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellBoostWordTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellBoostWordTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellBoostWordTask.stringdata0))
        return static_cast<void*>(this);
    return HunspellTask::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellBoostWordTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HunspellTask::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker_t {
    QByteArrayData data[1];
    char stringdata0[34];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker_t qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker = {
    {
QT_MOC_LITERAL(0, 0, 33) // "QtVirtualKeyboard::HunspellWo..."

    },
    "QtVirtualKeyboard::HunspellWorker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__HunspellWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QtVirtualKeyboard::HunspellWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QtVirtualKeyboard::HunspellWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker.data,
    qt_meta_data_QtVirtualKeyboard__HunspellWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QtVirtualKeyboard::HunspellWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::HunspellWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__HunspellWorker.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int QtVirtualKeyboard::HunspellWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
