#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>

namespace QmlCacheGeneratedCode {
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_AlternativeKeys_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardColumn_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_hi_IN_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_MultiSoundEffect_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_bg_BG_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fi_FI_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_NumberKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_es_ES_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_EnterKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_digits_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_it_IT_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_TraceInputArea_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fr_FR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_Keyboard_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_InputModeKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_ShiftKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_es_MX_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_uk_UA_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_HandwritingInputPanel_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_pt_BR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardLayoutLoader_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_he_IL_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_BackspaceKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardRow_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_WordCandidatePopupList_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_HideKeyboardKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_hu_HU_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_MultitapInputMethod_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_pt_PT_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_es_ES_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_ChangeLanguageKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_ModeKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_he_IL_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fr_CA_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ro_RO_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_dialpad_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_CharacterPreviewBubble_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_cs_CZ_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_pl_PL_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_hr_HR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_sl_SI_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_digits_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_BaseKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_tr_TR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_es_MX_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_hi_IN_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_vi_VN_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_sk_SK_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_vi_VN_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_styles_retro_style_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_HandwritingModeKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_el_GR_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_PopupList_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_TraceInputKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_SpaceKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_sr_SP_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_numbers_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_ShadowInputControl_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_Key_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_SelectionControl_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_et_EE_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_FillerKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_digits_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_ru_RU_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_da_DK_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_numbers_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_numbers_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_symbols_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_InputPanel_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_SymbolModeKey_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_nb_NO_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_sq_AL_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_styles_default_style_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_de_DE_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_layouts_sv_SE_main_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}
namespace _0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardLayout_qml { 
    extern const unsigned char qmlData[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), nullptr, nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/AlternativeKeys.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_AlternativeKeys_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/KeyboardColumn.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardColumn_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/hi_IN/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_hi_IN_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/MultiSoundEffect.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_MultiSoundEffect_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/bg_BG/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_bg_BG_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fi_FI/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fi_FI_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/NumberKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_NumberKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/es_ES/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_es_ES_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/EnterKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_EnterKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ar_AR/digits.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_digits_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/it_IT/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_it_IT_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/TraceInputArea.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_TraceInputArea_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fr_FR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fr_FR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/Keyboard.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_Keyboard_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/InputModeKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_InputModeKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/ShiftKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_ShiftKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/es_MX/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_es_MX_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/uk_UA/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_uk_UA_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/HandwritingInputPanel.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_HandwritingInputPanel_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fa_FA/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/pt_BR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_pt_BR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/KeyboardLayoutLoader.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardLayoutLoader_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/he_IL/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_he_IL_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/BackspaceKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_BackspaceKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/KeyboardRow.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardRow_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/WordCandidatePopupList.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_WordCandidatePopupList_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/HideKeyboardKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_HideKeyboardKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/hu_HU/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_hu_HU_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/MultitapInputMethod.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_MultitapInputMethod_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/pt_PT/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_pt_PT_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/es_ES/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_es_ES_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/ChangeLanguageKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_ChangeLanguageKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/ModeKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_ModeKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/he_IL/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_he_IL_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fr_CA/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fr_CA_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ro_RO/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ro_RO_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fallback/dialpad.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_dialpad_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/CharacterPreviewBubble.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_CharacterPreviewBubble_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/cs_CZ/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_cs_CZ_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/pl_PL/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_pl_PL_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/hr_HR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_hr_HR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/sl_SI/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_sl_SI_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fallback/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ar_AR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ar_AR/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fa_FA/digits.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_digits_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/BaseKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_BaseKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/tr_TR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_tr_TR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/es_MX/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_es_MX_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/hi_IN/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_hi_IN_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/vi_VN/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_vi_VN_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/sk_SK/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_sk_SK_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/vi_VN/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_vi_VN_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/styles/retro/style.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_styles_retro_style_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fallback/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/HandwritingModeKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_HandwritingModeKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/el_GR/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_el_GR_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/PopupList.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_PopupList_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/TraceInputKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_TraceInputKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/SpaceKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_SpaceKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/sr_SP/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_sr_SP_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ar_AR/numbers.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ar_AR_numbers_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/ShadowInputControl.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_ShadowInputControl_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/Key.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_Key_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/SelectionControl.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_SelectionControl_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/et_EE/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_et_EE_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/FillerKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_FillerKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fallback/digits.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_digits_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/ru_RU/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_ru_RU_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/da_DK/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_da_DK_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fallback/numbers.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fallback_numbers_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fa_FA/numbers.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_numbers_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/fa_FA/symbols.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_fa_FA_symbols_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/InputPanel.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_InputPanel_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/SymbolModeKey.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_SymbolModeKey_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/nb_NO/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_nb_NO_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/sq_AL/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_sq_AL_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/styles/default/style.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_styles_default_style_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/de_DE/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_de_DE_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/layouts/sv_SE/main.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_layouts_sv_SE_main_qml::unit);
        resourcePathToCachedUnit.insert(QStringLiteral("/QtQuick/VirtualKeyboard/content/components/KeyboardLayout.qml"), &QmlCacheGeneratedCode::_0x5f_QtQuick_VirtualKeyboard_content_components_KeyboardLayout_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.version = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_default_style)() {
    ::unitRegistry();
    Q_INIT_RESOURCE(content_styles_default_virtualkeyboard_default_style_qmlcache);
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_default_style))
int QT_MANGLE_NAMESPACE(qCleanupResources_virtualkeyboard_default_style)() {
    Q_CLEANUP_RESOURCE(content_styles_default_virtualkeyboard_default_style_qmlcache);
    return 1;
}
int QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_retro_style)() {
    ::unitRegistry();
    Q_INIT_RESOURCE(content_styles_retro_virtualkeyboard_retro_style_qmlcache);
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_retro_style))
int QT_MANGLE_NAMESPACE(qCleanupResources_virtualkeyboard_retro_style)() {
    Q_CLEANUP_RESOURCE(content_styles_retro_virtualkeyboard_retro_style_qmlcache);
    return 1;
}
int QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_content)() {
    ::unitRegistry();
    Q_INIT_RESOURCE(content_virtualkeyboard_content_qmlcache);
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_virtualkeyboard_content))
int QT_MANGLE_NAMESPACE(qCleanupResources_virtualkeyboard_content)() {
    Q_CLEANUP_RESOURCE(content_virtualkeyboard_content_qmlcache);
    return 1;
}
int QT_MANGLE_NAMESPACE(qInitResources_qmake_virtualkeyboard_layouts)() {
    ::unitRegistry();
    Q_INIT_RESOURCE(_rcc_qmake_virtualkeyboard_layouts_qmlcache);
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_qmake_virtualkeyboard_layouts))
int QT_MANGLE_NAMESPACE(qCleanupResources_qmake_virtualkeyboard_layouts)() {
    Q_CLEANUP_RESOURCE(_rcc_qmake_virtualkeyboard_layouts_qmlcache);
    return 1;
}
