/****************************************************************************
** Meta object code from reading C++ file 'qvirtualkeyboardtrace.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtvirtualkeyboard/src/virtualkeyboard/qvirtualkeyboardtrace.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvirtualkeyboardtrace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QVirtualKeyboardTrace_t {
    QByteArrayData data[26];
    char stringdata0[256];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QVirtualKeyboardTrace_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QVirtualKeyboardTrace_t qt_meta_stringdata_QVirtualKeyboardTrace = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QVirtualKeyboardTrace"
QT_MOC_LITERAL(1, 22, 14), // "traceIdChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 7), // "traceId"
QT_MOC_LITERAL(4, 46, 15), // "channelsChanged"
QT_MOC_LITERAL(5, 62, 13), // "lengthChanged"
QT_MOC_LITERAL(6, 76, 6), // "length"
QT_MOC_LITERAL(7, 83, 12), // "finalChanged"
QT_MOC_LITERAL(8, 96, 7), // "isFinal"
QT_MOC_LITERAL(9, 104, 15), // "canceledChanged"
QT_MOC_LITERAL(10, 120, 10), // "isCanceled"
QT_MOC_LITERAL(11, 131, 14), // "opacityChanged"
QT_MOC_LITERAL(12, 146, 7), // "opacity"
QT_MOC_LITERAL(13, 154, 6), // "points"
QT_MOC_LITERAL(14, 161, 3), // "pos"
QT_MOC_LITERAL(15, 165, 5), // "count"
QT_MOC_LITERAL(16, 171, 8), // "addPoint"
QT_MOC_LITERAL(17, 180, 5), // "point"
QT_MOC_LITERAL(18, 186, 14), // "setChannelData"
QT_MOC_LITERAL(19, 201, 7), // "channel"
QT_MOC_LITERAL(20, 209, 5), // "index"
QT_MOC_LITERAL(21, 215, 4), // "data"
QT_MOC_LITERAL(22, 220, 11), // "channelData"
QT_MOC_LITERAL(23, 232, 8), // "channels"
QT_MOC_LITERAL(24, 241, 5), // "final"
QT_MOC_LITERAL(25, 247, 8) // "canceled"

    },
    "QVirtualKeyboardTrace\0traceIdChanged\0"
    "\0traceId\0channelsChanged\0lengthChanged\0"
    "length\0finalChanged\0isFinal\0canceledChanged\0"
    "isCanceled\0opacityChanged\0opacity\0"
    "points\0pos\0count\0addPoint\0point\0"
    "setChannelData\0channel\0index\0data\0"
    "channelData\0channels\0final\0canceled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QVirtualKeyboardTrace[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       6,  134, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       4,    0,   87,    2, 0x06 /* Public */,
       5,    1,   88,    2, 0x06 /* Public */,
       7,    1,   91,    2, 0x06 /* Public */,
       9,    1,   94,    2, 0x06 /* Public */,
      11,    1,   97,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      13,    2,  100,    2, 0x02 /* Public */,
      13,    1,  105,    2, 0x22 /* Public | MethodCloned */,
      13,    0,  108,    2, 0x22 /* Public | MethodCloned */,
      16,    1,  109,    2, 0x02 /* Public */,
      18,    3,  112,    2, 0x02 /* Public */,
      22,    3,  119,    2, 0x02 /* Public */,
      22,    2,  126,    2, 0x22 /* Public | MethodCloned */,
      22,    1,  131,    2, 0x22 /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::QReal,   12,

 // methods: parameters
    QMetaType::QVariantList, QMetaType::Int, QMetaType::Int,   14,   15,
    QMetaType::QVariantList, QMetaType::Int,   14,
    QMetaType::QVariantList,
    QMetaType::Int, QMetaType::QPointF,   17,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::QVariant,   19,   20,   21,
    QMetaType::QVariantList, QMetaType::QString, QMetaType::Int, QMetaType::Int,   19,   14,   15,
    QMetaType::QVariantList, QMetaType::QString, QMetaType::Int,   19,   14,
    QMetaType::QVariantList, QMetaType::QString,   19,

 // properties: name, type, flags
       3, QMetaType::Int, 0x00495103,
      23, QMetaType::QStringList, 0x00495103,
       6, QMetaType::Int, 0x00495001,
      24, QMetaType::Bool, 0x00495103,
      25, QMetaType::Bool, 0x00495103,
      12, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,

       0        // eod
};

void QVirtualKeyboardTrace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QVirtualKeyboardTrace *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->traceIdChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->channelsChanged(); break;
        case 2: _t->lengthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->finalChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->canceledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->opacityChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 6: { QVariantList _r = _t->points((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QVariantList _r = _t->points((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 8: { QVariantList _r = _t->points();
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->addPoint((*reinterpret_cast< const QPointF(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setChannelData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3]))); break;
        case 11: { QVariantList _r = _t->channelData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 12: { QVariantList _r = _t->channelData((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 13: { QVariantList _r = _t->channelData((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QVirtualKeyboardTrace::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::traceIdChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardTrace::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::channelsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardTrace::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::lengthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardTrace::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::finalChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardTrace::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::canceledChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QVirtualKeyboardTrace::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QVirtualKeyboardTrace::opacityChanged)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QVirtualKeyboardTrace *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->traceId(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->channels(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->length(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isFinal(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isCanceled(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->opacity(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QVirtualKeyboardTrace *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTraceId(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setChannels(*reinterpret_cast< QStringList*>(_v)); break;
        case 3: _t->setFinal(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setCanceled(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setOpacity(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QVirtualKeyboardTrace::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QVirtualKeyboardTrace.data,
    qt_meta_data_QVirtualKeyboardTrace,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QVirtualKeyboardTrace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QVirtualKeyboardTrace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QVirtualKeyboardTrace.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QVirtualKeyboardTrace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QVirtualKeyboardTrace::traceIdChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QVirtualKeyboardTrace::channelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QVirtualKeyboardTrace::lengthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QVirtualKeyboardTrace::finalChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QVirtualKeyboardTrace::canceledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QVirtualKeyboardTrace::opacityChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
